@@ -21,7 +21,7 @@
 #endif
 #endif /* STANDALONE */
 
-#include "xmlversion.h"
+#include <libxml/xmlversion.h>
 
 #ifdef LIBXML_FTP_ENABLED
 #include <stdio.h>
@@ -63,6 +63,7 @@
 
 #include <libxml/xmlmemory.h>
 #include <libxml/nanoftp.h>
+#include <libxml/xmlerror.h>
 
 /* #define DEBUG_FTP 1  */
 #ifdef STANDALONE
@@ -71,6 +72,14 @@
 #endif
 #endif
 
+/**
+ * A couple portability macros
+ */
+#ifndef _WINSOCKAPI_
+#define closesocket(s) close(s)
+#define SOCKET int
+#endif
+
 static char hostname[100];
 
 #define FTP_COMMAND_OK		200
@@ -87,8 +96,8 @@ typedef struct xmlNanoFTPCtxt {
     char *passwd;	/* passwd string */
     struct sockaddr_in ftpAddr; /* the socket address struct */
     int passive;	/* currently we support only passive !!! */
-    int controlFd;	/* the file descriptor for the control socket */
-    int dataFd;		/* the file descriptor for the data socket */
+    SOCKET controlFd;	/* the file descriptor for the control socket */
+    SOCKET dataFd;	/* the file descriptor for the data socket */
     int state;		/* WRITE / READ / CLOSED */
     int returnValue;	/* the protocol return value */
     /* buffer for data received from the control connection */
@@ -116,10 +125,18 @@ static int proxyType = 0;	/* uses TYPE or a@b ? */
 void
 xmlNanoFTPInit(void) {
     const char *env;
+#ifdef _WINSOCKAPI_
+    WSADATA wsaData;    
+#endif
 
     if (initialized)
 	return;
 
+#ifdef _WINSOCKAPI_
+    if (WSAStartup(MAKEWORD(1, 1), &wsaData) != 0)
+	return;
+#endif
+
     gethostname(hostname, sizeof(hostname));
 
     proxyPort = 21;
@@ -167,6 +184,10 @@ xmlNanoFTPCleanup(void) {
 	proxyPasswd = NULL;
     }
     hostname[0] = 0;
+#ifdef _WINSOCKAPI_
+    if (initialized)
+	WSACleanup();
+#endif
     initialized = 0;
     return;
 }
@@ -400,9 +421,9 @@ xmlNanoFTPScanProxy(const char *URL) {
     }
 #ifdef DEBUG_FTP
     if (URL == NULL)
-	printf("Removing FTP proxy info\n");
+	xmlGenericError(xmlGenericErrorContext, "Removing FTP proxy info\n");
     else
-	printf("Using FTP proxy %s\n", URL);
+	xmlGenericError(xmlGenericErrorContext, "Using FTP proxy %s\n", URL);
 #endif
     if (URL == NULL) return;
     buf[index] = 0;
@@ -488,7 +509,7 @@ xmlNanoFTPFreeCtxt(void * ctx) {
     if (ctxt->protocol != NULL) xmlFree(ctxt->protocol);
     if (ctxt->path != NULL) xmlFree(ctxt->path);
     ctxt->passive = 1;
-    if (ctxt->controlFd >= 0) close(ctxt->controlFd);
+    if (ctxt->controlFd >= 0) closesocket(ctxt->controlFd);
     ctxt->controlFd = -1;
     ctxt->controlBufIndex = -1;
     ctxt->controlBufUsed = -1;
@@ -547,7 +568,8 @@ xmlNanoFTPGetMore(void *ctx) {
 
     if ((ctxt->controlBufIndex < 0) || (ctxt->controlBufIndex > FTP_BUF_SIZE)) {
 #ifdef DEBUG_FTP
-        printf("xmlNanoFTPGetMore : controlBufIndex = %d\n",
+        xmlGenericError(xmlGenericErrorContext,
+		"xmlNanoFTPGetMore : controlBufIndex = %d\n",
 		ctxt->controlBufIndex);
 #endif
 	return(-1);
@@ -555,14 +577,16 @@ xmlNanoFTPGetMore(void *ctx) {
 
     if ((ctxt->controlBufUsed < 0) || (ctxt->controlBufUsed > FTP_BUF_SIZE)) {
 #ifdef DEBUG_FTP
-        printf("xmlNanoFTPGetMore : controlBufUsed = %d\n",
+        xmlGenericError(xmlGenericErrorContext,
+		"xmlNanoFTPGetMore : controlBufUsed = %d\n",
 		ctxt->controlBufUsed);
 #endif
 	return(-1);
     }
     if (ctxt->controlBufIndex > ctxt->controlBufUsed) {
 #ifdef DEBUG_FTP
-        printf("xmlNanoFTPGetMore : controlBufIndex > controlBufUsed %d > %d\n",
+        xmlGenericError(xmlGenericErrorContext,
+		"xmlNanoFTPGetMore : controlBufIndex > controlBufUsed %d > %d\n",
 	       ctxt->controlBufIndex, ctxt->controlBufUsed);
 #endif
 	return(-1);
@@ -580,7 +604,8 @@ xmlNanoFTPGetMore(void *ctx) {
     size = FTP_BUF_SIZE - ctxt->controlBufUsed;
     if (size == 0) {
 #ifdef DEBUG_FTP
-        printf("xmlNanoFTPGetMore : buffer full %d \n", ctxt->controlBufUsed);
+        xmlGenericError(xmlGenericErrorContext,
+		"xmlNanoFTPGetMore : buffer full %d \n", ctxt->controlBufUsed);
 #endif
 	return(0);
     }
@@ -590,12 +615,13 @@ xmlNanoFTPGetMore(void *ctx) {
      */
     if ((len = recv(ctxt->controlFd, &ctxt->controlBuf[ctxt->controlBufIndex],
 		    size, 0)) < 0) {
-	close(ctxt->controlFd); ctxt->controlFd = -1;
+	closesocket(ctxt->controlFd); ctxt->controlFd = -1;
         ctxt->controlFd = -1;
         return(-1);
     }
 #ifdef DEBUG_FTP
-    printf("xmlNanoFTPGetMore : read %d [%d - %d]\n", len,
+    xmlGenericError(xmlGenericErrorContext,
+	    "xmlNanoFTPGetMore : read %d [%d - %d]\n", len,
 	   ctxt->controlBufUsed, ctxt->controlBufUsed + len);
 #endif
     ctxt->controlBufUsed += len;
@@ -631,7 +657,8 @@ get_more:
     end = &ctxt->controlBuf[ctxt->controlBufUsed];
 
 #ifdef DEBUG_FTP
-    printf("\n<<<\n%s\n--\n", ptr);
+    xmlGenericError(xmlGenericErrorContext,
+	    "\n<<<\n%s\n--\n", ptr);
 #endif
     while (ptr < end) {
         cur = xmlNanoFTPParseResponse(ctxt, ptr, end - ptr);
@@ -661,11 +688,11 @@ get_more:
     ctxt->controlBufIndex = ptr - ctxt->controlBuf;
 #ifdef DEBUG_FTP
     ptr = &ctxt->controlBuf[ctxt->controlBufIndex];
-    printf("\n---\n%s\n--\n", ptr);
+    xmlGenericError(xmlGenericErrorContext, "\n---\n%s\n--\n", ptr);
 #endif
 
 #ifdef DEBUG_FTP
-    printf("Got %d\n", res);
+    xmlGenericError(xmlGenericErrorContext, "Got %d\n", res);
 #endif
     return(res / 100);
 }
@@ -741,7 +768,7 @@ xmlNanoFTPSendUser(void *ctx) {
     buf[sizeof(buf) - 1] = 0;
     len = strlen(buf);
 #ifdef DEBUG_FTP
-    printf(buf);
+    xmlGenericError(xmlGenericErrorContext, buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
     if (res < 0) return(res);
@@ -774,7 +801,7 @@ xmlNanoFTPSendPasswd(void *ctx) {
     buf[sizeof(buf) - 1] = 0;
     len = strlen(buf);
 #ifdef DEBUG_FTP
-    printf(buf);
+    xmlGenericError(xmlGenericErrorContext, buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
     if (res < 0) return(res);
@@ -801,7 +828,7 @@ xmlNanoFTPQuit(void *ctx) {
     sprintf(buf, "QUIT\r\n");
     len = strlen(buf);
 #ifdef DEBUG_FTP
-    printf(buf);
+    xmlGenericError(xmlGenericErrorContext, buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
     return(0);
@@ -861,7 +888,7 @@ xmlNanoFTPConnect(void *ctx) {
      */
     if (connect(ctxt->controlFd, (struct sockaddr *) &ctxt->ftpAddr,
                 sizeof(struct sockaddr_in)) < 0) {
-        close(ctxt->controlFd); ctxt->controlFd = -1;
+        closesocket(ctxt->controlFd); ctxt->controlFd = -1;
         ctxt->controlFd = -1;
 	return(-1);
     }
@@ -871,7 +898,7 @@ xmlNanoFTPConnect(void *ctx) {
      */
     res = xmlNanoFTPGetResponse(ctxt);
     if (res != 2) {
-        close(ctxt->controlFd); ctxt->controlFd = -1;
+        closesocket(ctxt->controlFd); ctxt->controlFd = -1;
         ctxt->controlFd = -1;
 	return(-1);
     }
@@ -927,11 +954,11 @@ xmlNanoFTPConnect(void *ctx) {
             buf[sizeof(buf) - 1] = 0;
             len = strlen(buf);
 #ifdef DEBUG_FTP
-	    printf(buf);
+	    xmlGenericError(xmlGenericErrorContext, buf);
 #endif
 	    res = send(ctxt->controlFd, buf, len, 0);
 	    if (res < 0) {
-		close(ctxt->controlFd);
+		closesocket(ctxt->controlFd);
 		ctxt->controlFd = -1;
 	        return(res);
 	    }
@@ -957,17 +984,17 @@ xmlNanoFTPConnect(void *ctx) {
                     buf[sizeof(buf) - 1] = 0;
                     len = strlen(buf);
 #ifdef DEBUG_FTP
-		    printf(buf);
+		    xmlGenericError(xmlGenericErrorContext, buf);
 #endif
 		    res = send(ctxt->controlFd, buf, len, 0);
 		    if (res < 0) {
-			close(ctxt->controlFd);
+			closesocket(ctxt->controlFd);
 			ctxt->controlFd = -1;
 			return(res);
 		    }
 		    res = xmlNanoFTPGetResponse(ctxt);
 		    if (res > 3) {
-			close(ctxt->controlFd);
+			closesocket(ctxt->controlFd);
 			ctxt->controlFd = -1;
 			return(-1);
 		    }
@@ -978,7 +1005,7 @@ xmlNanoFTPConnect(void *ctx) {
 		case 5:
 		case -1:
 		default:
-		    close(ctxt->controlFd);
+		    closesocket(ctxt->controlFd);
 		    ctxt->controlFd = -1;
 		    return(-1);
 	    }
@@ -1001,11 +1028,11 @@ xmlNanoFTPConnect(void *ctx) {
                 buf[sizeof(buf) - 1] = 0;
                 len = strlen(buf);
 #ifdef DEBUG_FTP
-		printf(buf);
+		xmlGenericError(xmlGenericErrorContext, buf);
 #endif
 		res = send(ctxt->controlFd, buf, len, 0);
 		if (res < 0) {
-		    close(ctxt->controlFd); ctxt->controlFd = -1;
+		    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 		    ctxt->controlFd = -1;
 		    return(res);
 		}
@@ -1016,7 +1043,7 @@ xmlNanoFTPConnect(void *ctx) {
 		    break;
 		}    
 		if (proxyType == 1) {
-		    close(ctxt->controlFd); ctxt->controlFd = -1;
+		    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 		    ctxt->controlFd = -1;
 		    return(-1);
 		}
@@ -1040,11 +1067,11 @@ xmlNanoFTPConnect(void *ctx) {
                 buf[sizeof(buf) - 1] = 0;
                 len = strlen(buf);
 #ifdef DEBUG_FTP
-		printf(buf);
+		xmlGenericError(xmlGenericErrorContext, buf);
 #endif
 		res = send(ctxt->controlFd, buf, len, 0);
 		if (res < 0) {
-		    close(ctxt->controlFd); ctxt->controlFd = -1;
+		    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 		    ctxt->controlFd = -1;
 		    return(res);
 		}
@@ -1069,11 +1096,11 @@ xmlNanoFTPConnect(void *ctx) {
                 buf[sizeof(buf) - 1] = 0;
                 len = strlen(buf);
 #ifdef DEBUG_FTP
-		printf(buf);
+		xmlGenericError(xmlGenericErrorContext, buf);
 #endif
 		res = send(ctxt->controlFd, buf, len, 0);
 		if (res < 0) {
-		    close(ctxt->controlFd); ctxt->controlFd = -1;
+		    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 		    ctxt->controlFd = -1;
 		    return(res);
 		}
@@ -1084,7 +1111,7 @@ xmlNanoFTPConnect(void *ctx) {
 		    return(0);
 		}
 		if (proxyType == 2) {
-		    close(ctxt->controlFd); ctxt->controlFd = -1;
+		    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 		    ctxt->controlFd = -1;
 		    return(-1);
 		}
@@ -1094,7 +1121,7 @@ xmlNanoFTPConnect(void *ctx) {
 		 * send the code or at least the sequence in use.
 		 */
 	    default:
-		close(ctxt->controlFd); ctxt->controlFd = -1;
+		closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 		ctxt->controlFd = -1;
 		return(-1);
 	}
@@ -1104,7 +1131,7 @@ xmlNanoFTPConnect(void *ctx) {
      */
     res = xmlNanoFTPSendUser(ctxt);
     if (res < 0) {
-        close(ctxt->controlFd); ctxt->controlFd = -1;
+        closesocket(ctxt->controlFd); ctxt->controlFd = -1;
         ctxt->controlFd = -1;
 	return(-1);
     }
@@ -1119,13 +1146,13 @@ xmlNanoFTPConnect(void *ctx) {
 	case 5:
         case -1:
 	default:
-	    close(ctxt->controlFd); ctxt->controlFd = -1;
+	    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 	    ctxt->controlFd = -1;
 	    return(-1);
     }
     res = xmlNanoFTPSendPasswd(ctxt);
     if (res < 0) {
-        close(ctxt->controlFd); ctxt->controlFd = -1;
+        closesocket(ctxt->controlFd); ctxt->controlFd = -1;
         ctxt->controlFd = -1;
 	return(-1);
     }
@@ -1134,13 +1161,14 @@ xmlNanoFTPConnect(void *ctx) {
 	case 2:
 	    break;
 	case 3:
-	    fprintf(stderr, "FTP server asking for ACCNT on anonymous\n");
+	    xmlGenericError(xmlGenericErrorContext,
+		    "FTP server asking for ACCNT on anonymous\n");
 	case 1:
 	case 4:
 	case 5:
         case -1:
 	default:
-	    close(ctxt->controlFd); ctxt->controlFd = -1;
+	    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 	    ctxt->controlFd = -1;
 	    return(-1);
     }
@@ -1210,7 +1238,7 @@ xmlNanoFTPCwd(void *ctx, char *directory) {
     buf[sizeof(buf) - 1] = 0;
     len = strlen(buf);
 #ifdef DEBUG_FTP
-    printf(buf);
+    xmlGenericError(xmlGenericErrorContext, buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
     if (res < 0) return(res);
@@ -1244,15 +1272,13 @@ xmlNanoFTPGetConnection(void *ctx) {
     unsigned char ad[6], *adp, *portp;
     unsigned int temp[6];
     struct sockaddr_in dataAddr;
-#ifdef HAVE_SOCKLEN_T
-    socklen_t dataAddrLen;
-#else
-    unsigned int dataAddrLen;
-#endif
+    SOCKLEN_T dataAddrLen;
 
     ctxt->dataFd = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
     if (ctxt->dataFd < 0) {
-        fprintf(stderr, "xmlNanoFTPGetConnection: failed to create socket\n");
+        xmlGenericError(xmlGenericErrorContext,
+		"xmlNanoFTPGetConnection: failed to create socket\n");
+	return(-1);
     }
     dataAddrLen = sizeof(dataAddr);
     memset(&dataAddr, 0, dataAddrLen);
@@ -1262,33 +1288,34 @@ xmlNanoFTPGetConnection(void *ctx) {
 	sprintf(buf, "PASV\r\n");
         len = strlen(buf);
 #ifdef DEBUG_FTP
-	printf(buf);
+	xmlGenericError(xmlGenericErrorContext, buf);
 #endif
 	res = send(ctxt->controlFd, buf, len, 0);
 	if (res < 0) {
-	    close(ctxt->dataFd); ctxt->dataFd = -1;
+	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(res);
 	}
         res = xmlNanoFTPReadResponse(ctx);
 	if (res != 2) {
 	    if (res == 5) {
-	        close(ctxt->dataFd); ctxt->dataFd = -1;
+	        closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 		return(-1);
 	    } else {
 		/*
 		 * retry with an active connection
 		 */
-	        close(ctxt->dataFd); ctxt->dataFd = -1;
+	        closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	        ctxt->passive = 0;
 	    }
 	}
 	cur = &ctxt->controlBuf[ctxt->controlBufAnswer]; 
 	while (((*cur < '0') || (*cur > '9')) && *cur != '\0') cur++;
-	if (sscanf(cur, "%d,%d,%d,%d,%d,%d", &temp[0], &temp[1], &temp[2],
+	if (sscanf(cur, "%u,%u,%u,%u,%u,%u", &temp[0], &temp[1], &temp[2],
 	            &temp[3], &temp[4], &temp[5]) != 6) {
-	    fprintf(stderr, "Invalid answer to PASV\n");
+	    xmlGenericError(xmlGenericErrorContext,
+		    "Invalid answer to PASV\n");
 	    if (ctxt->dataFd != -1) {
-		close(ctxt->dataFd); ctxt->dataFd = -1;
+		closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    }
 	    return(-1);
 	}
@@ -1296,24 +1323,27 @@ xmlNanoFTPGetConnection(void *ctx) {
 	memcpy(&dataAddr.sin_addr, &ad[0], 4);
 	memcpy(&dataAddr.sin_port, &ad[4], 2);
 	if (connect(ctxt->dataFd, (struct sockaddr *) &dataAddr, dataAddrLen) < 0) {
-	    fprintf(stderr, "Failed to create a data connection\n");
-	    close(ctxt->dataFd); ctxt->dataFd = -1;
+	    xmlGenericError(xmlGenericErrorContext,
+		    "Failed to create a data connection\n");
+	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return (-1);
 	}
     } else {
         getsockname(ctxt->dataFd, (struct sockaddr *) &dataAddr, &dataAddrLen);
 	dataAddr.sin_port = 0;
 	if (bind(ctxt->dataFd, (struct sockaddr *) &dataAddr, dataAddrLen) < 0) {
-	    fprintf(stderr, "Failed to bind a port\n");
-	    close(ctxt->dataFd); ctxt->dataFd = -1;
+	    xmlGenericError(xmlGenericErrorContext,
+		    "Failed to bind a port\n");
+	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return (-1);
 	}
         getsockname(ctxt->dataFd, (struct sockaddr *) &dataAddr, &dataAddrLen);
 
 	if (listen(ctxt->dataFd, 1) < 0) {
-	    fprintf(stderr, "Could not listen on port %d\n",
+	    xmlGenericError(xmlGenericErrorContext,
+		    "Could not listen on port %d\n",
 	            ntohs(dataAddr.sin_port));
-	    close(ctxt->dataFd); ctxt->dataFd = -1;
+	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return (-1);
 	}
 	adp = (unsigned char *) &dataAddr.sin_addr;
@@ -1330,17 +1360,17 @@ xmlNanoFTPGetConnection(void *ctx) {
         buf[sizeof(buf) - 1] = 0;
         len = strlen(buf);
 #ifdef DEBUG_FTP
-	printf(buf);
+	xmlGenericError(xmlGenericErrorContext, buf);
 #endif
 
 	res = send(ctxt->controlFd, buf, len, 0);
 	if (res < 0) {
-	    close(ctxt->dataFd); ctxt->dataFd = -1;
+	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(res);
 	}
         res = xmlNanoFTPGetResponse(ctxt);
 	if (res != 2) {
-	    close(ctxt->dataFd); ctxt->dataFd = -1;
+	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(-1);
         }
     }
@@ -1364,7 +1394,7 @@ xmlNanoFTPCloseConnection(void *ctx) {
     fd_set rfd, efd;
     struct timeval tv;
 
-    close(ctxt->dataFd); ctxt->dataFd = -1;
+    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
     tv.tv_sec = 15;
     tv.tv_usec = 0;
     FD_ZERO(&rfd);
@@ -1376,18 +1406,19 @@ xmlNanoFTPCloseConnection(void *ctx) {
 #ifdef DEBUG_FTP
 	perror("select");
 #endif
-	close(ctxt->controlFd); ctxt->controlFd = -1;
+	closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 	return(-1);
     }
     if (res == 0) {
 #ifdef DEBUG_FTP
-	fprintf(stderr, "xmlNanoFTPCloseConnection: timeout\n");
+	xmlGenericError(xmlGenericErrorContext,
+		"xmlNanoFTPCloseConnection: timeout\n");
 #endif
-	close(ctxt->controlFd); ctxt->controlFd = -1;
+	closesocket(ctxt->controlFd); ctxt->controlFd = -1;
     } else {
 	res = xmlNanoFTPGetResponse(ctxt);
 	if (res != 2) {
-	    close(ctxt->controlFd); ctxt->controlFd = -1;
+	    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 	    return(-1);
 	}
     }
@@ -1567,16 +1598,16 @@ xmlNanoFTPList(void *ctx, ftpListCallback callback, void *userData,
     buf[sizeof(buf) - 1] = 0;
     len = strlen(buf);
 #ifdef DEBUG_FTP
-    printf(buf);
+    xmlGenericError(xmlGenericErrorContext, buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
     if (res < 0) {
-	close(ctxt->dataFd); ctxt->dataFd = -1;
+	closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	return(res);
     }
     res = xmlNanoFTPReadResponse(ctxt);
     if (res != 1) {
-	close(ctxt->dataFd); ctxt->dataFd = -1;
+	closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	return(-res);
     }
 
@@ -1592,29 +1623,29 @@ xmlNanoFTPList(void *ctx, ftpListCallback callback, void *userData,
 #ifdef DEBUG_FTP
 	    perror("select");
 #endif
-	    close(ctxt->dataFd); ctxt->dataFd = -1;
+	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(-1);
 	}
 	if (res == 0) {
 	    res = xmlNanoFTPCheckResponse(ctxt);
 	    if (res < 0) {
-		close(ctxt->dataFd); ctxt->dataFd = -1;
+		closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 		ctxt->dataFd = -1;
 		return(-1);
 	    }
 	    if (res == 2) {
-		close(ctxt->dataFd); ctxt->dataFd = -1;
+		closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 		return(0);
 	    }
 
 	    continue;
 	}
 
-	if ((len = read(ctxt->dataFd, &buf[index], sizeof(buf) - (index + 1))) < 0) {
+	if ((len = recv(ctxt->dataFd, &buf[index], sizeof(buf) - (index + 1), 0)) < 0) {
 #ifdef DEBUG_FTP
-	    perror("read");
+	    perror("recv");
 #endif
-	    close(ctxt->dataFd); ctxt->dataFd = -1;
+	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    ctxt->dataFd = -1;
 	    return(-1);
 	}
@@ -1661,16 +1692,16 @@ xmlNanoFTPGetSocket(void *ctx, const char *filename) {
     sprintf(buf, "TYPE I\r\n");
     len = strlen(buf);
 #ifdef DEBUG_FTP
-    printf(buf);
+    xmlGenericError(xmlGenericErrorContext, buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
     if (res < 0) {
-	close(ctxt->dataFd); ctxt->dataFd = -1;
+	closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	return(res);
     }
     res = xmlNanoFTPReadResponse(ctxt);
     if (res != 2) {
-	close(ctxt->dataFd); ctxt->dataFd = -1;
+	closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	return(-res);
     }
     if (filename == NULL)
@@ -1688,16 +1719,16 @@ xmlNanoFTPGetSocket(void *ctx, const char *filename) {
     buf[sizeof(buf) - 1] = 0;
     len = strlen(buf);
 #ifdef DEBUG_FTP
-    printf(buf);
+    xmlGenericError(xmlGenericErrorContext, buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
     if (res < 0) {
-	close(ctxt->dataFd); ctxt->dataFd = -1;
+	closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	return(res);
     }
     res = xmlNanoFTPReadResponse(ctxt);
     if (res != 1) {
-	close(ctxt->dataFd); ctxt->dataFd = -1;
+	closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	return(-res);
     }
     return(ctxt->dataFd);
@@ -1742,26 +1773,26 @@ xmlNanoFTPGet(void *ctx, ftpDataCallback callback, void *userData,
 #ifdef DEBUG_FTP
 	    perror("select");
 #endif
-	    close(ctxt->dataFd); ctxt->dataFd = -1;
+	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(-1);
 	}
 	if (res == 0) {
 	    res = xmlNanoFTPCheckResponse(ctxt);
 	    if (res < 0) {
-		close(ctxt->dataFd); ctxt->dataFd = -1;
+		closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 		ctxt->dataFd = -1;
 		return(-1);
 	    }
 	    if (res == 2) {
-		close(ctxt->dataFd); ctxt->dataFd = -1;
+		closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 		return(0);
 	    }
 
 	    continue;
 	}
-	if ((len = read(ctxt->dataFd, &buf, sizeof(buf))) < 0) {
+	if ((len = recv(ctxt->dataFd, buf, sizeof(buf), 0)) < 0) {
 	    callback(userData, buf, len);
-	    close(ctxt->dataFd); ctxt->dataFd = -1;
+	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(-1);
 	}
 	callback(userData, buf, len);
@@ -1791,9 +1822,9 @@ xmlNanoFTPRead(void *ctx, void *dest, int len) {
     if (dest == NULL) return(-1);
     if (len <= 0) return(0);
 
-    len = read(ctxt->dataFd, dest, len);
+    len = recv(ctxt->dataFd, dest, len, 0);
 #ifdef DEBUG_FTP
-    printf("Read %d bytes\n", len);
+    xmlGenericError(xmlGenericErrorContext, "Recvd %d bytes\n", len);
 #endif
     if (len <= 0) {
 	xmlNanoFTPCloseConnection(ctxt);
@@ -1850,12 +1881,12 @@ xmlNanoFTPClose(void *ctx) {
 	return(-1);
 
     if (ctxt->dataFd >= 0) {
-	close(ctxt->dataFd);
+	closesocket(ctxt->dataFd);
 	ctxt->dataFd = -1;
     }
     if (ctxt->controlFd >= 0) {
 	xmlNanoFTPQuit(ctxt);
-	close(ctxt->controlFd);
+	closesocket(ctxt->controlFd);
 	ctxt->controlFd = -1;
     }
     xmlNanoFTPFreeCtxt(ctxt);
@@ -1871,7 +1902,8 @@ xmlNanoFTPClose(void *ctx) {
 void ftpList(void *userData, const char *filename, const char* attrib,
 	     const char *owner, const char *group, unsigned long size, int links,
 	     int year, const char *month, int day, int hour, int minute) {
-    printf("%s %s %s %ld %s\n", attrib, owner, group, size, filename);
+    xmlGenericError(xmlGenericErrorContext,
+	    "%s %s %s %ld %s\n", attrib, owner, group, size, filename);
 }
 void ftpData(void *userData, const char *data, int len) {
     if (userData == NULL) return;
@@ -1891,7 +1923,8 @@ int main(int argc, char **argv) {
     if (argc > 1) {
 	ctxt = xmlNanoFTPNewCtxt(argv[1]);
 	if (xmlNanoFTPConnect(ctxt) < 0) {
-	    fprintf(stderr, "Couldn't connect to %s\n", argv[1]);
+	    xmlGenericError(xmlGenericErrorContext,
+		    "Couldn't connect to %s\n", argv[1]);
 	    exit(1);
 	}
 	if (argc > 2)
@@ -1899,14 +1932,16 @@ int main(int argc, char **argv) {
     } else
 	ctxt = xmlNanoFTPConnectTo("localhost", 0);
     if (ctxt == NULL) {
-        fprintf(stderr, "Couldn't connect to localhost\n");
+        xmlGenericError(xmlGenericErrorContext,
+		"Couldn't connect to localhost\n");
         exit(1);
     }
     xmlNanoFTPList(ctxt, ftpList, NULL, tstfile);
     output = fopen("/tmp/tstdata", "w");
     if (output != NULL) {
 	if (xmlNanoFTPGet(ctxt, ftpData, (void *) output, tstfile) < 0)
-	    fprintf(stderr, "Failed to get file\n");
+	    xmlGenericError(xmlGenericErrorContext,
+		    "Failed to get file\n");
 	
     }
     xmlNanoFTPClose(ctxt);
@@ -1918,7 +1953,8 @@ int main(int argc, char **argv) {
 #ifdef STANDALONE
 #include <stdio.h>
 int main(int argc, char **argv) {
-    printf("%s : FTP support not compiled in\n", argv[0]);
+    xmlGenericError(xmlGenericErrorContext,
+	    "%s : FTP support not compiled in\n", argv[0]);
     return(0);
 }
 #endif /* STANDALONE */
