@@ -12,7 +12,7 @@
 #include "config.h"
 #endif
 
-#include "xmlversion.h"
+#include <libxml/xmlversion.h>
 #if defined(LIBXML_XPATH_ENABLED) && defined(LIBXML_DEBUG_ENABLED)
 
 #include <stdio.h>
@@ -40,8 +40,15 @@
 #include <libxml/parser.h>
 #include <libxml/debugXML.h>
 #include <libxml/xmlmemory.h>
-
+#include <libxml/parserInternals.h>
+#include <libxml/xpathInternals.h>
+#include <libxml/xmlerror.h>
+#if defined(LIBXML_XPTR_ENABLED)
+#include <libxml/xpointer.h>
+static int xptr = 0;
+#endif
 static int debug = 0;
+static int valid = 0;
 static int expr = 0;
 static xmlDocPtr document = NULL;
 
@@ -79,69 +86,26 @@ static xmlChar buffer[] =
 </EXAMPLE>\n\
 ";
 
-void xmlXPAthDebugDumpNodeSet(FILE *output, xmlNodeSetPtr cur) {
-    int i;
-
-    if (cur == NULL) {
-	fprintf(output, "NodeSet is NULL !\n");
-	return;
-        
-    }
-
-    fprintf(output, "Set contains %d nodes:\n", cur->nodeNr);
-    for (i = 0;i < cur->nodeNr;i++) {
-        fprintf(output, "%d", i + 1);
-	if (cur->nodeTab[i] == NULL)
-	    fprintf(output, " NULL\n");
-	else if ((cur->nodeTab[i]->type == XML_DOCUMENT_NODE) ||
-	         (cur->nodeTab[i]->type == XML_HTML_DOCUMENT_NODE))
-	    fprintf(output, " /\n");
-	else if (cur->nodeTab[i]->type == XML_ATTRIBUTE_NODE)
-	    xmlDebugDumpAttr(output, (xmlAttrPtr)cur->nodeTab[i], 2);
-	else
-	    xmlDebugDumpOneNode(output, cur->nodeTab[i], 2);
-    }
-}
-
-void xmlXPAthDebugDumpObject(FILE *output, xmlXPathObjectPtr cur) {
-    if (cur == NULL) {
-        fprintf(output, "Object is empty (NULL)\n");
-	return;
-    }
-    switch(cur->type) {
-        case XPATH_UNDEFINED:
-	    fprintf(output, "Object is uninitialized\n");
-	    break;
-        case XPATH_NODESET:
-	    fprintf(output, "Object is a Node Set :\n");
-	    xmlXPAthDebugDumpNodeSet(output, cur->nodesetval);
-	    break;
-        case XPATH_BOOLEAN:
-	    fprintf(output, "Object is a Boolean : ");
-	    if (cur->boolval) fprintf(output, "true\n");
-	    else fprintf(output, "false\n");
-	    break;
-        case XPATH_NUMBER:
-	    fprintf(output, "Object is a number : %0g\n", cur->floatval);
-	    break;
-        case XPATH_STRING:
-	    fprintf(output, "Object is a string : ");
-	    xmlDebugDumpString(output, cur->stringval);
-	    fprintf(output, "\n");
-	    break;
-    }
-}
 
 void testXPath(const char *str) {
     xmlXPathObjectPtr res;
     xmlXPathContextPtr ctxt;
     
-    ctxt = xmlXPathNewContext(document);
-    if (expr)
-	res = xmlXPathEvalExpression(BAD_CAST str, ctxt);
-    else
-	res = xmlXPathEval(BAD_CAST str, ctxt);
-    xmlXPAthDebugDumpObject(stdout, res);
+#if defined(LIBXML_XPTR_ENABLED)
+    if (xptr) {
+	ctxt = xmlXPtrNewContext(document, NULL, NULL);
+	res = xmlXPtrEval(BAD_CAST str, ctxt);
+    } else {
+#endif
+	ctxt = xmlXPathNewContext(document);
+	if (expr)
+	    res = xmlXPathEvalExpression(BAD_CAST str, ctxt);
+	else
+	    res = xmlXPathEval(BAD_CAST str, ctxt);
+#if defined(LIBXML_XPTR_ENABLED)
+    }
+#endif
+    xmlXPathDebugDumpObject(stdout, res, 0);
     xmlXPathFreeObject(res);
     xmlXPathFreeContext(ctxt);
 }
@@ -149,14 +113,25 @@ void testXPath(const char *str) {
 void testXPathFile(const char *filename) {
     FILE *input;
     char expr[5000];
+    int len;
 
     input = fopen(filename, "r");
     if (input == NULL) {
-        fprintf(stderr, "Cannot open %s for reading\n", filename);
+        xmlGenericError(xmlGenericErrorContext,
+		"Cannot open %s for reading\n", filename);
 	return;
     }
-    while (fscanf(input, "%s", expr) != EOF) {
-        testXPath(expr);
+    while (fgets(expr, 4500, input) != NULL) {
+	len = strlen(expr);
+	len--;
+	while ((len >= 0) && 
+	       ((expr[len] == '\n') || (expr[len] == '\t') ||
+		(expr[len] == '\r') || (expr[len] == ' '))) len--;
+	expr[len + 1] = 0;      
+	if (len >= 0) {
+	    printf("\n========================\nExpression: %s\n", expr) ;
+	    testXPath(expr);
+	}
     }
 
     fclose(input);
@@ -169,8 +144,14 @@ int main(int argc, char **argv) {
     char *filename = NULL;
 
     for (i = 1; i < argc ; i++) {
+#if defined(LIBXML_XPTR_ENABLED)
+	if ((!strcmp(argv[i], "-xptr")) || (!strcmp(argv[i], "--xptr")))
+	    xptr++;
+#endif
 	if ((!strcmp(argv[i], "-debug")) || (!strcmp(argv[i], "--debug")))
 	    debug++;
+	if ((!strcmp(argv[i], "-valid")) || (!strcmp(argv[i], "--valid")))
+	    valid++;
 	if ((!strcmp(argv[i], "-expr")) || (!strcmp(argv[i], "--expr")))
 	    expr++;
 	if ((!strcmp(argv[i], "-i")) || (!strcmp(argv[i], "--input")))
@@ -178,6 +159,7 @@ int main(int argc, char **argv) {
 	if ((!strcmp(argv[i], "-f")) || (!strcmp(argv[i], "--file")))
 	    usefile++;
     }
+    if (valid != 0) xmlDoValidityCheckingDefaultValue = 1;
     if (document == NULL) {
         if (filename == NULL)
 	    document = xmlParseDoc(buffer);
@@ -201,6 +183,10 @@ int main(int argc, char **argv) {
 	       argv[0]);
 	printf("\tParse the XPath strings and output the result of the parsing\n");
 	printf("\t--debug : dump a debug version of the result\n");
+	printf("\t--valid : switch on DTD support in the parser\n");
+#if defined(LIBXML_XPTR_ENABLED)
+	printf("\t--xptr : expressions are XPointer expressions\n");
+#endif
 	printf("\t--expr : debug XPath expressions only\n");
 	printf("\t--input filename : or\n");
 	printf("\t-i filename      : read the document from filename\n");
